# Dumitru Cotorobai - (c) 2021
import sys
from datetime import datetime
import tweepy
import pandas as pd
import re
from array import *

def GetLocationEndPoint(TweetString, start):
    #Common delimeters in the specific location substring
    end = TweetString.find(' while ')
    if(end == -1):
        end = TweetString.find(' due ')
        if(end  == -1):
            end = TweetString.find('.')
            if(end < start):
                end = len(TweetString)
    
    return end

def GetTTCTweetLocation(TweetString):
    
    #Set Initial Location
    location = ''

    #Remove TTC Update substring
    if((TweetString.find('UPDATE:') > -1) or (TweetString.find('Update:') > -1)):
        TweetString = TweetString[8:-1]
    
    #Start at String Index 0
    start = 0
    end = 0
    
    #Remove Additional Spaces of Substring
    for i in range(0, len(TweetString)):
        if(TweetString[i] != ' '):
            TweetString = TweetString[i:]
            break

    
    #Look for Keyword in Tweet
    keywords = [' at ', ' between ', ' near ', ' via ', ' from ']

    for keyword in keywords:
        
        match = TweetString.find(keyword)
        if(match > -1):
            if keyword == ' between ':
                start = TweetString.find(' between ') + 9
                end = GetLocationEndPoint(TweetString = TweetString, start = start)
                break
            
            if keyword == ' at ':
                start = TweetString.find(' at ') + 4
                end = GetLocationEndPoint(TweetString = TweetString, start = start)
                break

            if keyword == ' near ':
                start = TweetString.find(' near ') + 6
                end = GetLocationEndPoint(TweetString = TweetString, start = start)
                break

            if keyword == ' via ':
                start = TweetString.find(' via ') + 5
                end = GetLocationEndPoint(TweetString = TweetString, start = start)
                break
            
            if keyword == ' from ':
                start = TweetString.find(' from ') + 6
                end = GetLocationEndPoint(TweetString = TweetString, start = start)
                break
    
    #Check if a specific location is found
    if(start < end):
        SpecificLocation = TweetString[start : end]
    else:
        SpecificLocation = ''

    #Get Main Route Path
    keywords = [r'continues to', r':']
    for keyword in keywords:
        end = TweetString.find(keyword)
        if(end > -1):
            if keyword == keywords[1]:
                location = TweetString[0 : end]
            else:
                location = TweetString[0 : end - 1] 
            break



    #Did not find an exact location thus return route
    if SpecificLocation == '':
        return location            
    return location + ' : ' + SpecificLocation



def GetTTCTweetStatus(TweetString):
    keywords = ['Delay', 'Resume', 'No Service', 'Detour', 'Collision']

    #Look for Keywords in Tweet
    for keyword in keywords:
        match = re.search(keyword, TweetString, re.IGNORECASE)
        if match:
            match = re.search(r'\d\d minutes', TweetString, re.IGNORECASE)
            if match:
                return (keyword + ': ' + match.group())
            else:
                return keyword
    
    return ''

def GetTTCStatus(CONSUMER_KEY, CONSUMER_SECRET, ACCESS_TOKEN, ACCESS_TOKEN_SECRET):
    
    #Connect to API
    auth = tweepy.OAuthHandler(CONSUMER_KEY, CONSUMER_SECRET)
    auth.set_access_token(ACCESS_TOKEN, ACCESS_TOKEN_SECRET)
    api = tweepy.API(auth)

    #User Information
    username = r'TTCnotices'
    count = 50
    page = 1

    #GetTweets 
    i = 1
    j = 1
    tweets_index = []
    tweets_delay = []
    tweets_delaytime = []
    tweets_location = []
    tweets_date = []
    tweets_text = []

    while True:
        if(i <= page):

            tweets = api.user_timeline(id = username, count = count, page = page, tweet_mode = 'extended')
            for tweet in tweets:
                #Print to console Index, Time, and Tweet
                #print(str(j) + '. \t{' + str(tweet.created_at) + ' }\t' + tweet.full_text)
                tweets_index.append(j)

                
                #Get Time Zone Right
                time = tweet.created_at.strftime('%H')
                time = int(time)
                time = (time - 5)%24
                time = str(time)
                #tweets_date.append(tweet.created_at.strftime(time + ':%M:%S'))
                tweets_date.append(tweet.created_at.strftime('%m/%d/%Y, '+ time + ':%M:%S'))

                tweets_delay.append(GetTTCTweetStatus(tweet.full_text))
                if(tweets_delay[-1] == ''):
                    tweets_location.append('')
                else:
                    tweets_location.append(GetTTCTweetLocation(tweet.full_text))
                tweets_text.append(tweet.full_text)

                #Increment Tweet Index
                j += 1
            
            #Increment Page Index
            i += 1
        else:
            break

    #Store Info to Excel Spreadsheet
    #d = {'Index' : tweets_index, 'Date' : tweets_date, 'Status' : tweets_delay, 'Location': tweets_location, 'Text' : tweets_text}
    #df = pd.DataFrame(data = d, columns = ['Index', 'Date', 'Status', 'Location', 'Text'])
    #df.to_excel('Book1.xlsx')

    #Object to return to main
    ReturnObject = []

    #Return time, status flag, and location
    for i in range(0, len(tweets_delay) - 1):
        if tweets_delay[i] != '':
            ReturnObject.append([tweets_date[i], tweets_delay[i], tweets_location[i]])
    
    return ReturnObject




# Samir Mitha - (c) 2021
import pyowm
import requests, json

def fetch_weather(city, country, owm_API_key):
	"""Gets the current temperature and weather conditions from the
	city and country provided

    Parameters
    ----------
    city : str
        The city name of the location
    country: str
    	The country name of the location
    owm_API_key: str
    	The OpenWeatherMap API key to call on own functions

    Returns
    -------
    temp: int
        The temperature in degrees Celsius
    status: str
    	The current weather conditions as a short string
    """
	owm = pyowm.OWM(owm_API_key)
	manager = owm.weather_manager()
	observation = manager.weather_at_place(city + ', ' + country)
	w = observation.weather
	temperature = w.temperature('celsius')
	status = w.detailed_status
	temp = int(temperature['temp'])
	return temp, status

def weather_multiplier(status):
	"""Gets a multiplier to apply to the travel time, based on the current
	time and weather conditions

    Parameters
    ----------
    status : str
        The current weather conditions as a short string

    Returns
    -------
    multiplier: float
    	The multiplier of the current weather condition
    """
	switcher1 = {
		"thunderstorm with light rain":1.041,
		"thunderstorm with rain":1.041,
		"thunderstorm with heavy rain":1.105,
		"light thunderstorm":1,
		"thunderstorm":1,
		"heavy thunderstorm":1,
		"ragged thunderstorm":1,
		"thunderstorm with light drizzle":1.041,
		"thunderstorm with drizzle":1.041,
		"thunderstorm with heavy drizzle":1.105,
		"light intensity drizzle":1,
		"drizzle":1,
		"heavy intensity drizzle":1.041,
		"light intensity drizzle rain":1,
		"drizzle rain":1.041,
		"heavy intensity drizzle rain":1.041,
		"shower rain and drizzle":1.041,
		"heavy shower rain and drizzle":1.105,
		"shower drizzle":1.041,
		"light rain":1.041,
		"moderate rain":1.041,
		"heavy intensity rain":1.105,
		"very heavy rain":1.105,
		"extreme rain":1.105,
		"freezing rain":1.105,
		"light intensity shower rain":1,
		"shower rain":1.041,
		"heavy intensity shower rain":1.041,
		"ragged shower rain":1.041,
		"light snow":1.195,
		"snow":1.195,
		"heavy snow":1.986,
		"sleet":1.195,
		"light shower sleet":1.195,
		"shower sleet":1.195,
		"light rain and snow":1.195,
		"rain and snow":1.195,
		"light shower snow":1.195,
		"shower snow":1.195,
		"heavy shower snow":1.986
	}
	switcher2 = {
		"thunderstorm with light rain":1.2754,
		"thunderstorm with rain":1.2754,
		"thunderstorm with heavy rain":1.6324,
		"light thunderstorm":1,
		"thunderstorm":1,
		"heavy thunderstorm":1,
		"ragged thunderstorm":1,
		"thunderstorm with light drizzle":1.2754,
		"thunderstorm with drizzle":1.2754,
		"thunderstorm with heavy drizzle":1.6324,
		"light intensity drizzle":1,
		"drizzle":1,
		"heavy intensity drizzle":1.2754,
		"light intensity drizzle rain":1,
		"drizzle rain":1.2754,
		"heavy intensity drizzle rain":1.2754,
		"shower rain and drizzle":1.2754,
		"heavy shower rain and drizzle":1.6324,
		"shower drizzle":1.2754,
		"light rain":1.2754,
		"moderate rain":1.2754,
		"heavy intensity rain":1.6324,
		"very heavy rain":1.6324,
		"extreme rain":1.6324,
		"freezing rain":1.6324,
		"light intensity shower rain":1,
		"shower rain":1.2754,
		"heavy intensity shower rain":1.2754,
		"ragged shower rain":1.2754,
		"light snow":1.4428,
		"snow":1.4428,
		"heavy snow":2.6242,
		"sleet":1.4428,
		"light shower sleet":1.4428,
		"shower sleet":1.4428,
		"light rain and snow":1.4428,
		"rain and snow":1.4428,
		"light shower snow":1.4428,
		"shower snow":1.4428,
		"heavy shower snow":2.6242
	}
	switcher3 = {
		"thunderstorm with light rain":1.07,
		"thunderstorm with rain":1.07,
		"thunderstorm with heavy rain":1.255,
		"light thunderstorm":1,
		"thunderstorm":1,
		"heavy thunderstorm":1,
		"ragged thunderstorm":1,
		"thunderstorm with light drizzle":1.07,
		"thunderstorm with drizzle":1.07,
		"thunderstorm with heavy drizzle":1.255,
		"light intensity drizzle":1,
		"drizzle":1,
		"heavy intensity drizzle":1.07,
		"light intensity drizzle rain":1,
		"drizzle rain":1.07,
		"heavy intensity drizzle rain":1.07,
		"shower rain and drizzle":1.07,
		"heavy shower rain and drizzle":1.255,
		"shower drizzle":1.07,
		"light rain":1.07,
		"moderate rain":1.07,
		"heavy intensity rain":1.255,
		"very heavy rain":1.255,
		"extreme rain":1.255,
		"freezing rain":1.255,
		"light intensity shower rain":1,
		"shower rain":1.07,
		"heavy intensity shower rain":1.07,
		"ragged shower rain":1.07,
		"light snow":1.136,
		"snow":1.136,
		"heavy snow":1.59,
		"sleet":1.136,
		"light shower sleet":1.136,
		"shower sleet":1.136,
		"light rain and snow":1.136,
		"rain and snow":1.136,
		"light shower snow":1.136,
		"shower snow":1.136,
		"heavy shower snow":1.59
	}
	switcher4 = {
		"thunderstorm with light rain":1.143,
		"thunderstorm with rain":1.143,
		"thunderstorm with heavy rain":1.3348,
		"light thunderstorm":1,
		"thunderstorm":1,
		"heavy thunderstorm":1,
		"ragged thunderstorm":1,
		"thunderstorm with light drizzle":1.143,
		"thunderstorm with drizzle":1.143,
		"thunderstorm with heavy drizzle":1.3348,
		"light intensity drizzle":1,
		"drizzle":1,
		"heavy intensity drizzle":1.143,
		"light intensity drizzle rain":1,
		"drizzle rain":1.143,
		"heavy intensity drizzle rain":1.143,
		"shower rain and drizzle":1.143,
		"heavy shower rain and drizzle":1.3348,
		"shower drizzle":1.143,
		"light rain":1.143,
		"moderate rain":1.143,
		"heavy intensity rain":1.3348,
		"very heavy rain":1.3348,
		"extreme rain":1.3348,
		"freezing rain":1.3348,
		"light intensity shower rain":1,
		"shower rain":1.143,
		"heavy intensity shower rain":1.143,
		"ragged shower rain":1.143,
		"light snow":1.19,
		"snow":1.19,
		"heavy snow":1.5254,
		"sleet":1.19,
		"light shower sleet":1.19,
		"shower sleet":1.19,
		"light rain and snow":1.19,
		"rain and snow":1.19,
		"light shower snow":1.19,
		"shower snow":1.19,
		"heavy shower snow":1.5254
	}
	current_time = datetime.datetime.now()
	if current_time.hour >= 0 and current_time.hour < 6:
		return switcher1.get(status, 1)
	if current_time.hour >= 19 and current_time.hour <= 23:
		return switcher1.get(status, 1)
	if current_time.hour >= 6 and current_time.hour < 9:
		return switcher2.get(status, 1)
	if current_time.hour >= 9 and current_time.hour < 15:
		return switcher3.get(status, 1)
	if current_time.hour >= 15 and current_time.hour < 19:
		return switcher4.get(status, 1)


# Samir Mitha - (c) 2021
import StatusTTC as ttc
import StatusYRT as yrt

def ttc_tweet_short(CONSUMER_KEY, CONSUMER_SECRET, ACCESS_TOKEN, ACCESS_TOKEN_SECRET):
	"""Shortens tweet informations and splits into categories

    Parameters
    ----------
    CONSUMER_KEY : str
    	tweepy API key
    CONSUMER_SECRET : str
    	tweepy API key
    ACCESS_TOKEN : str
    	tweepy API key
    ACCESS_TOKEN_SECRET : str
    	tweepy API key

    Returns
    -------
    tweets: list
        list of tweet objects
    """
	status = ttc.GetTTCStatus(CONSUMER_KEY, CONSUMER_SECRET, ACCESS_TOKEN, ACCESS_TOKEN_SECRET)
	j = 0
	for i in status:
		j = j + 1
		date = (i[0].split(', '))[0]
		time = (i[0].split(', '))[1]
		condition_full = i[1].split(': ')
		if len(condition_full) > 1:
			condition = (i[1].split(': '))[0]
			condition_time = int((i[1].split(': '))[1].replace(' minutes', ''))/60
		else:
			condition = (i[1].split(': '))[0]
			condition_time = None
		full_route =(i[2].split(' : '))
		route = full_route[0]
		if len(full_route) > 1:
			if " and " in full_route[1]:
				start = (full_route[1].split(' and '))[0]
				end = (full_route[1].split(' and '))[1]
			elif " to " in full_route[1]:
				start = (full_route[1].split(' to '))[0]
				end = (full_route[1].split(' to '))[1]
			else:
				start = full_route[1]
				end = None
		else:
			pass
		tweet = [j, date, time, condition, condition_time, route, start, end]
		# tweet object contains:
		# j -> tweet number
		# date -> tweet date
		# time -> tweet time
		# condition -> traffic status condition
		# condition_time -> estimated duration of condition (in minutes)
		# route -> TTC line number
		# start -> start location of transit condition
		# end -> end location of transit condition
		tweets[i] = tweet
	return(tweets)

def yrt_tweet_short(CONSUMER_KEY, CONSUMER_SECRET, ACCESS_TOKEN, ACCESS_TOKEN_SECRET):
	"""Shortens tweet informations and splits into categories

    Parameters
    ----------
    CONSUMER_KEY : str
    	tweepy API key
    CONSUMER_SECRET : str
    	tweepy API key
    ACCESS_TOKEN : str
    	tweepy API key
    ACCESS_TOKEN_SECRET : str
    	tweepy API key

    Returns
    -------
    tweets: list
        list of tweet objects
    """
    status = yrt.GetTTCStatus(CONSUMER_KEY, CONSUMER_SECRET, ACCESS_TOKEN, ACCESS_TOKEN_SECRET)
    j = 0
    for i in status:
		j = j + 1
		date = (i[0].split(', '))[0]
		time = (i[0].split(', '))[1]
		condition_full = i[1].split(': ')
		if len(condition_full) > 1:
			condition = (i[1].split(': '))[0]
			condition_time = int((i[1].split(': '))[1].replace(' minutes', ''))/60
		else:
			condition = (i[1].split(': '))[0]
			condition_time = None
		full_route =(i[2].split(' : '))
		route = full_route[0]
		if len(full_route) > 1:
			if " and " in full_route[1]:
				start = (full_route[1].split(' and '))[0]
				end = (full_route[1].split(' and '))[1]
			elif " to " in full_route[1]:
				start = (full_route[1].split(' to '))[0]
				end = (full_route[1].split(' to '))[1]
			else:
				start = full_route[1]
				end = None
		else:
			start = None
			end = None
			pass
		tweet = [j, date, time, condition, condition_time, route, start, end]
		# tweet object contains:
		# j -> tweet number
		# date -> tweet date
		# time -> tweet time
		# condition -> traffic status condition
		# condition_time -> estimated duration of condition (in minutes)
		# route -> TTC line number
		# start -> start location of transit condition
		# end -> end location of transit condition
		tweets[i] = tweet
	return(tweets)


# Samir Mitha - (c) 2021
import googlemaps
import requests, json

def fetch_distance(origins, destinations, gmaps_API_key):
	"""Gets the distance and travel time between and origin and destination

    Parameters
    ----------
    origins : str
    	Address
    destination: str
    	Address
    gmaps_API_key: str
    	The GoogleMaps API key to call on googlemaps functions

    Returns
    -------
    distance: int
        The travel distance in meters
    travel_time: int
    	The travel time in seconds
    """
	gmaps = googlemaps.Client(key=gmaps_API_key)
	# required parameters
	origins = origins.replace(' ', '+')
	destinations = destinations.replace(' ', '+')
	# optional parameters
	mode = mode.replace(' ', '+')
	avoid = avoid.replace(' ', '+')
	units = units.replace(' ', '+')
	arrival_time = arrival_time.replace(' ', '+')
	departure_time = departure_time.replace(' ', '+')
	traffic_model = traffic_model.replace(' ', '+')
	transit_mode = transit_mode.replace(' ', '+')
	transit_routing_preference = transit_routing_preference.replace(' ', '+')
	# creating request
	url = 'https://maps.googleapis.com/maps/api/distancematrix/json?'
	# units
	if units == 'imperial':
		url_units = 'units=' + units
	else:
		url_units = 'units=metric'
	# origins
	url_origins = '&origins=' + origins
	# destinations
	url_destinations = '&destinations=' + destinations
	# mode
	if mode != '':
		url_mode = '&mode=' + mode
	else:
		url_mode = ''
	# avoid
	if avoid != '':
		url_avoid = '&avoid=' + avoid
	else:
		url_avoid = ''
	# arrival_time
	if arrival_time != '':
		url_arrival_time = '&arrival_time=' + arrival_time
	else:
		url_arrival_time = ''
	# departure_time
	if departure_time != '':
		url_departure_time = '&departure_time=' + departure_time
	else:
		url_departure_time = ''
	# traffic_model
	if traffic_model != '':
		url_traffic_model = '&traffic_model=' + traffic_model
	else:
		url_traffic_model = ''
	# transit_mode
	if transit_mode != '':
		url_transit_mode = '&transit_mode=' + transit_mode
	else:
		url_transit_mode = ''
	# transit_routing_preference
	if transit_routing_preference != '':
		url_transit_routing_preference = '&transit_routing_preference=' + transit_routing_preference
	else:
		url_transit_routing_preference = ''
	# creating url
	r = requests.get(url + url_units + url_origins + url_destinations + url_mode + url_avoid + url_arrival_time + url_departure_time + url_traffic_model + url_transit_mode + url_transit_routing_preference + '&key=' + gmaps_API_key)
	x = r.json()
	distance = int(x['rows'][0]['elements'][0]['distance']['value'])
	travel_time = int(x['rows'][0]['elements'][0]['duration']['value'])
	return distance, travel_time

def fetch_directions(origins, destinations, gmaps_API_key):
	"""Gets the line number and agency of public transit required

    Parameters
    ----------
    origins : str
    	Address
    destination: str
    	Address
    gmaps_API_key: str
    	The GoogleMaps API key to call on googlemaps functions

    Returns
    -------
    short_line: list
        A list of public transit agency line numbers
    agency: list
    	A list of public transit agency names
    """
	gmaps = googlemaps.Client(key=gmaps_API_key)
	# required parameters
	origins = origins.replace(' ', '+')
	destinations = destinations.replace(' ', '+')
	url = 'https://maps.googleapis.com/maps/api/directions/json?'
	r = requests.get(url + url_origins + url_destinations + '&key=' + gmaps_API_key)
	r = requests.get(url)
	x = r.json()
	num = len(x['routes'][0]['legs'][0]['steps'])
	for i in range(num):
		if x['routes'][0]['legs'][0]['steps'][i]['travel_mode'] == 'TRANSIT':
			short_line[i] = x['routes'][0]['legs'][0]['steps'][i]['transit_details']['line']['short_name']
			agency[i] = x['routes'][0]['legs'][0]['steps'][i]['transit_details']['line']['agencies'][0]['name']
	return(short_line, agency)


import setuptools

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'pypi.md'), encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name="notifyme_traveltimer",
    version="1.0.2",
    license="MIT",
    author="Samir Mitha, Dumitru Cotorobai",
    author_email="samir.mitha@gmail.ca",
    description="Functions for use with end to end travel and wait timing.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url = "https://github.com/SamirMitha/NotifyMe",
    packages=setuptools.find_packages(),
    install_requires = [
        'googlemaps',
        'pyowm',
        'tweepy',
        'pandas'
    ],
    python_requires=">=3.8",
)